# Installation

Executer `composer require dedi/sylius-settings-plugin`

Ajouter le plugin au fichier `config/bundles.php`:

# Configuration de base

Importer la configuration :

```yaml
# config/packages/dedi_sylius_settings_plugin.yaml

imports:
    - { resource: "@DediSyliusSettingsPlugin/config/config.yml" }
```

Définir la règle de sécurité pour protéger la section settings. La nouvelle règle doit être déclarée avant la règle admin :

```yaml
# config/packages/security.yaml
security:
    ...
    access_control:
        ...
+       - { path: "%sylius.security.admin_regex%/settings/edit/dedi.sylius_plugins_settings", role: CAN_CONFIGURE_PLUGINS }
        - { path: "%sylius.security.admin_regex%", role: ROLE_ADMINISTRATION_ACCESS }
```

# Plugin configuration

## Implémenter la configuration côté Plugin

Dans le plugin à installer et rendre compatible avec le `DediSettingsPlugin`, il est nécessaire d'implémenter un layer de configuration. 

Cette configuration sera rédigée en YAML côté plugin. En effet, on essaie d'avoir un minimum de dépendences non essentielles sur nos plugins. 

Le service `Settings` sera considéré comme la configuration par défaut du plugin. 

Pour commencer, ajouter le repository composer Dedi au composer.json : 

```json
{
    "repositories": [
        {
            "type": "composer",
            "url": "https://packages.bydedi.com"
        }
    ]
}
```

Ensuite, ajouter les contracts comme dépendence du plugin : 

```bash
  composer require dedi/sylius-plugin-settings-contracts:^1.0
```

Note : dans l'infra Docker interne pour nos plugin, il faut se mettre dans le bon dossier : 

```
make docker-bash CONTAINER=php
cd ../../
```

Implémenter une classe `Settings` et une interface `SettingsInterface` qui vont représenter la configuration du plugin : 

```php
<?php

namespace Dedi\AcmePlugin\DependencyInjection\Settings;

use Dedi\Contracts\SyliusPluginSettings\SettingsInterface as SettingsContractsInterface;

interface SettingsInterface extends SettingsContractsInterface
{
}
```

```php
<?php

declare(strict_types=1);

namespace Dedi\AcmePlugin\DependencyInjection\Settings;

class Settings implements SettingsInterface
{
    public function __construct(
        private readonly bool $enabled,
    )
    {
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }
}
```

De base, l'interface `SettingsInterface` du Contract n'implémente qu'un seul paramètre : `enabled`. 

Celui-ci servira à déterminer si le plugin est actif ou non. 

Il est néanmoins possible d'ajouter tout type de paramètre nécessaire à la configuration du plugin.


```php
<?php

namespace Dedi\AcmePlugin\DependencyInjection\Settings;

use Dedi\Contracts\SyliusPluginSettings\SettingsInterface as SettingsContractsInterface;

interface SettingsInterface extends SettingsContractsInterface
{
    public function isFoo(): bool;
    
    public function getBar(): string;
    
    public function getBaz(): array;
}
```

```php
<?php

declare(strict_types=1);

namespace Dedi\AcmePlugin\DependencyInjection\Settings;

class Settings implements SettingsInterface
{
    public function __construct(
        private readonly bool $enabled,
        private readonly bool $foo,
        private readonly string $bar,
        private readonly array $baz,
    )
    {
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }
    
    public function isFoo(): bool
    {
        return $this->foo;
    }
    
    public function getBar(): string
    {
        return $this->bar;
    }
    
    public function getBaz(): array
    {
        return $this->baz;
    }
}
```

Implémenter ensuite un `SettingsProvider` qui va permettre de récupérer la configuration du plugin : 

```php
<?php

declare(strict_types=1);

namespace Dedi\AcmePlugin\DependencyInjection\Settings;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\AcmePlugin\DependencyInjection\Settings\SettingsInterface;

class SettingsProvider implements SettingsProviderInterface
{
    public function __construct(
        private readonly SettingsInterface $settings,
    )
    {

    }

    public function getName(): string
    {
        return 'dedi_acme_plugin';
    }

    public function provide(bool $default = false): SettingsInterface
    {
        return $this->settings;
    }
}
```

Ici, il est important de définir le nom de la configuration dans la fonction `getName`, en respectant le format habituel. Vendor (dedi), plugin (acme_plugin), en snake_case.

Enfin, on configure nos deux services dans le fichier `services.yaml` : 

```yaml
services:
    dedi_sylius_acme_plugin.settings:
        class: Dedi\AcmePlugin\DependencyInjection\Settings\Settings
        arguments:
            $enabled: ~
            $foo: ~
            $bar: ~
            $baz: ~

    dedi_sylius_acme_plugin.settings_provider:
        class: Dedi\AcmePlugin\DependencyInjection\Settings\SettingsProvider
        arguments:
            - '@dedi_acme_plugin.settings'
```

De cette manière, l'objet `Settings` sera injecté dans le `SettingsProvider` qui sera utilisé par le `DediSettingsPlugin` pour récupérer la configuration du plugin.

Dans la configuration, on ne passe pas de valeurs pour les arguments : l'injection des valeurs sera réalisée à la prochaine étape. 

Dans la classe `Configuration` de notre plugin, on va définir la structure de configuration du plugin (voir <a href="https://symfony.com/doc/current/components/config/definition.html">la documentation officielle Symfony</a>).

```php
<?php

declare(strict_types=1);

namespace Dedi\AcmePlugin\DependencyInjection;

use Symfony\Component\Config\Definition\Builder\TreeBuilder;

class Configuration implements ConfigurationInterface
{
    public function getConfigTreeBuilder(): TreeBuilder
    {
        $treeBuilder = new TreeBuilder('dedi_acme_plugin');
        $rootNode = $treeBuilder->getRootNode();

        $rootNode
            ->children()
                ->booleanNode('enabled')->defaultFalse()->end()
                ->booleanNode('foo')->defaultFalse()->end()
                ->scalarNode('bar')->defaultValue('')->end()
                ->arrayNode('baz')->scalarPrototype()->end()
            ->end()
        ;

        return $treeBuilder;
    }
}
```

De cette manière, on pourra récupérer la configuration du plugin dans le fichier `config/packages/dedi_acme_plugin.yaml` :

```yaml
dedi_acme_plugin:
    enabled: true
    foo: true
    bar: 'baz'
    baz:
        - 'val1'
        - 'val2'
```

Ensuite, on va injecter la configuration dans le `Settings` :

```yaml
<?php

declare(strict_types=1);

namespace Dedi\AcmePlugin\DependencyInjection;

use Symfony\Component\DependencyInjection\Loader\YamlFileLoader;

class DediSyliusAcmeExtension extends AbstractResourceExtension
{
    public function load(array $config, ContainerBuilder $container): void
    {
        $config = $this->processConfiguration($this->getConfiguration([], $container), $config);
        $loader = new YamlFileLoader($container, new FileLocator(__DIR__ . '/../Resources/config'));
        $loader->load('services.yml');

        $settingsDefinition = $container->getDefinition('dedi_sylius_acme.settings');
        $settingsDefinition->replaceArgument('$enabled', $config['enabled']);
        $settingsDefinition->replaceArgument('$foo', $config['foo']);
        $settingsDefinition->replaceArgument('$bar', $config['bar']);
        $settingsDefinition->replaceArgument('$baz', $config['baz']);
    }
}
```

Ici, lors du chargement du container, on récupère la configuration du plugin et on injecte les valeurs dans le service `dedi_sylius_acme.settings`.

On peut dès lors utiliser cette configuration (<b>toujours en passant par le `SettingsProvider`</b>) dès lors que c'est nécessaire, notamment pour activer ou désactiver les fonctionnalités (c'est le minimum attendu) ou pour enrichir l'expérience utilisateur. 

<b>La règle indispensable à respecter en toutes circonstances, c'est de toujours utiliser le `SettingsProvider` pour récupérer la configuration du plugin.</b>

### Exemples d'utilisation de la configuration

#### Extension Twig

Définir l'extension Twig

```php
<?php

declare(strict_types=1);

namespace Dedi\SyliusAcmePlugin\DependencyInjection\Settings\Twig;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusAcmePlugin\DependencyInjection\Settings\SettingsInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

class SettingsCheckerExtension extends AbstractExtension
{
    public function __construct(private readonly SettingsProviderInterface $settingsProvider)
    {

    }

    public function getFunctions(): array
    {
        return [
            new TwigFunction('dedi_sylius_acme_check', [$this, 'check']),
            new TwigFunction('dedi_sylius_acme_is_enabled', [$this, 'isEnabled']),
        ];
    }

    public function check(string $setting): mixed
    {
        $settings = $this->settingsProvider->provide();
        
        Assert::isInstanceOf($settings, SettingsInterface::class);
        
        switch ($setting) {
            case 'enabled':
                return $settings->isEnabled();
            case 'foo':
                return $settings->isFoo();
            case 'bar':
                return $settings->getBar();
            case 'baz':
                return $settings->getBaz();                
            default:
                throw new \InvalidArgumentException(sprintf('Unrecognized Acme Plugin setting %s', $setting));
        }
    }

    public function isEnabled(): bool
    {
        return (bool) $this->check('enabled');
    }
}
```

Enregistrer l'extension dans le container

```yaml
services:
    dedi_sylius_acme_plugin.twig.settings_checker_extension:
        class: Dedi\SyliusAcmePlugin\DependencyInjection\Settings\Twig\SettingsCheckerExtension
        arguments:
            $settingsProvider: '@dedi_sylius_acme_plugin.settings_provider'
        tags:
            - { name: twig.extension }
```

Utiliser l'extension dans les templates Twig

```twig
{% if dedi_sylius_acme_is_enabled() %}
    <div>
        <p>Plugin Acme is enabled</p>
    </div>
{% endif %}
```

#### Injection dans un service

Par exemple, pour ajouter ou non l'entrée dans le menu admin en fonction de l'état activé ou désactivé du plugin : 

```php
<?php

declare(strict_types=1);

namespace Dedi\SyliusAcmePlugin\Menu;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Knp\Menu\ItemInterface;
use Sylius\Bundle\UiBundle\Menu\Event\MenuBuilderEvent;

final class AdminMenuListener
{
    public function __construct(private readonly SettingsProviderInterface $settingsProvider)
    {
    }
    public function addAdminMenuItems(MenuBuilderEvent $event): void
    {
        if (!$this->settingsProvider->provide()->isEnabled()) {
            return;
        }
        $menu = $event->getMenu();
        
        $submenu = $menu->getChild('main');
        $item = $submenu instanceof ItemInterface ? $submenu : $menu->getFirstChild();
        $item
            ->addChild('acme', [
                'route' => 'dedi_sylius_acme_plugin_admin_index',
            ])
    }
}
```

Enregistrer le listener dans le container en injectant le `SettingsProvider` :

```yaml
services:
    dedi_sylius_acme_plugin.menu.admin_menu_listener:
        class: Dedi\SyliusAcmePlugin\Menu\AdminMenuListener
        arguments:
            $settingsProvider: '@dedi_sylius_acme_plugin.settings_provider'
        tags:
            - { name: kernel.event_listener, event: sylius.menu.admin.main, method: addAdminMenuItems }
```



## Implémenter la configuration côté Projet

Notre projet va avoir deux dépendences : 
    - le `DediSettingsPlugin`, déjà installé, qui va permettre de récupérer la configuration des plugins et la rendre modifiable en Back Office
    - le `DediSyliusAcmePlugin`, qui est le plugin que nous venons de créer et qu'on va rendre configurable en Back Office

Le plugin `DediSyliusAcmePlugin` embarquera sa configuration, et mettra à disposition un service `Settings` hydraté à partir de la configuration YAML du plugin, ainsi qu'un service `SettingsProvider` qui permettra de récupérer la configuration du plugin.

En utilisant l'implémentation conjointement avec le `DediSettingsPlugin`, on pourra récupérer la configuration du plugin dans le Back Office.

Le plugin `DediSettingsPlugin` va décorer automatique le `SettingsProvider` pour aller exploiter le `SettingsProvider` du plugin MonsieurBizSettingsPlugin, et surcharger la configuration YAML avec la configuration en base de données.

Il va par ailleurs automatiquement enregistrer le plugin pour le rendre activable ou désactivable en Back Office, et gérer les droits d'accès à la configuration du plugin, ainsi qu'au fonctionnalités du plugin. 

Dans la configuration du `DediSyliusSettingsPlugin`, on va déclarer le plugin `DediSyliusAcmePlugin` :

```yaml
# config/packages/dedi_sylius_settings_plugin.yaml
dedi_sylius_settings:
    plugins:
        dedi_sylius_acme:
            model: Dedi\SyliusAcmePlugin\DependencyInjection\Settings\Settings 
            provider: dedi_sylius_acme.settings.provider
            keys:
                - enabled
                - foo
                - bar
                - baz
```

On utilise la clé `dedi_sylius_acme` pour déclarer la configuration du plugin. Cette clé doit être équivalente au résultat de la fonction `getName` du `SettingsProvider` du plugin.

On défini par ailleurs les clés de configuration du plugin, qui correspondent aux propriétés définies dans le `SettingsInterface` du plugin.

Enfin, on défini le model en fournissant le namespace de la classe `Settings` du plugin, ainsi que le service `SettingsProvider` du plugin.

Attention : le service `SettingsProvider` du plugin doit être enregistré dans le container avec le nom `dedi_sylius_acme.settings.provider`. Si vous utilisez une déclaration par nom de classe, il est nécessaire de respecter ce nom.

On peut par ailleurs déclarer une factory, même si la factory générique fournie par le `DediSettingsPlugin` est suffisante pour la plupart des cas d'usage.

On va ensuite configurer le plugin `M̀onsieurBizSettingsPlugin` pour qu'il prenne en charge la configuration du plugin `DediSyliusAcmePlugin` :

```yaml
# config/packages/monsieurbiz_settings_plugin.yaml
monsieurbiz_sylius_settings:
    plugins:
        app.dedi_acme_plugin:
            vendor_name: Dedi
            vendor_url: https://www.dedi-agency.com/
            plugin_name: Acme
            description: Gestion du plugin Acme
            icon: ~
            use_locales: true
            classes:
                form: App\Configuration\Settings\Type\AcmeSettingsType
```

On va implémenter la classe décrivant le formulaire de configuration du plugin, `App\Configuration\Settings\Type\AcmeSettingsType` :

```php
<?php

declare(strict_types=1);

namespace App\Configuration\Settings\Type;

use Dedi\SyliusAcmePlugin\DependencyInjection\Settings\SettingsInterface;
use MonsieurBiz\SyliusSettingsPlugin\Form\AbstractSettingsType;
use MonsieurBiz\SyliusSettingsPlugin\Form\SettingsTypeInterface;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\FormBuilderInterface;
use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Webmozart\Assert\Assert;

class VoucherSettingsType extends AbstractSettingsType implements SettingsTypeInterface
{
    public function __construct(private readonly SettingsProviderInterface $settingsProvider)
    {

    }
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $acmePluginSettings = $this->settingsProvider->provide();

        Assert::isInstanceOf($acmePluginSettings, SettingsInterface::class);

        $this->addWithDefaultCheckbox(
            $builder,
            'foo',
            CheckboxType::class,
            [
                'label' => 'dedi.settings.dedi_acme_plugin.foo',
                'required' => false,
                'data' => $acmePluginSettings->isFoo()
            ]
        );
        $this->addWithDefaultText(
            $builder,
            'bar',
            InputType::class,
            [
                'label' => 'dedi.settings.dedi_acme_plugin.bar',
                'required' => false,
                'data' => $acmePluginSettings->getBar()
            ]
        );
        $this->addWithDefaultText(
            $builder,
            'baz',
            CollectionType::class,
            [
                'label' => 'dedi.settings.dedi_acme_plugin.baz',
                'required' => false,
                'data' => $acmePluginSettings->getBaz()
            ]
        );
        
    }
}
```

#TODO
App\Configuration\Settings\Type\VoucherSettingsType:
class: 'App\Configuration\Settings\Type\VoucherSettingsType'
arguments:
$settingsProvider: '@dedi_sylius_voucher.settings.provider'
tags:
- { name: form.type }


On défini chaque champs de configuration, en passant comme `data` la valeur récupérée via le `SettingsProvider` du plugin.

La clé de configuration `enabled` n'est pas ajoutée : elle est gérée nativement par le plugin `DediSettingsPlugin` dans l'interface d'activation / désactivation des plugins. 

Et... c'est tout ! 

Désormais, vous avez en Back Office : 
- une section Réglage, avec les configurations de Sylius
- dans cette section, une section Plugins, avec la liste des plugins installés et la possibilité de les activer / désactiver
- une section par plugin, avec les clés de configuration qui vont bien
- la configuration du plugin est récupérée en Base de Données et vient surcharger la configuration YAML du plugin, utilisée par défaut

### Securité

Le plugin `DediSettingsPlugin` gère nativement les droits d'accès à la configuration des plugins. 

D'abord, il restreint l'accès à l'interface d'activation / désactivation des plugins à certains rôles uniquement. 

Dans le fichier de configuration du plugin, on peut définir les rôles qui ont accès à la configuration des plugins : 

```yaml
# config/packages/dedi_sylius_settings_plugin.yaml
dedi_sylius_settings:
    roles: [ROLE_ADMINISTRATION_ACCESS]
```

Si l'utilisateur a au moins un des rôles listés, il aura accès à la configuration des plugins.

Ensuite, le plugin gère automatiquement les permissions de configuration et d'accès aux différents plugins. 

Si un plugin est désactivé, il ne s'affichera pas dans l'interface Réglages du Back Office. 

De la même manière, si on essaie d'accéder à une URL du plugin (exemple : /admin/acme/index) et que le plugin est désactivé, une erreur 403: Access Denied sera retournée.

<b>Attention ! pour que l'accès soit restreint aux URLs du plugin, il est nécessaire d'activer les permissions sur l'URL (à faire dans le routing du plugin) !</b>

Exemple : 

```yaml
dedi_sylius_acme_plugin_admin_index:
    path: /acme/index
    methods: [GET]
    defaults:
        _controller: Dedi\SyliusAcmePlugin\Controller\Admin\AcmeController::index
        _sylius:
            template: '@DediSyliusAcmePlugin/Admin/index.html.twig'
            permission: true
```

La clé de configuration `permission` permet de surcharger le nom du domaine de permissions utilisée pour restreindre l'accès aux URLs du plugin.
La clé de configuration par défaut est le nom du plugin en snake_case, mais il est possible de surcharger cette clé pour utiliser une permission personnalisée.

```yaml
dedi_sylius_settings:
    plugins:
        dedi_sylius_acme:
            ...
            permission: dedi_acme
            ...
```
